/*
 * $Id: cf_main.c,v 1.1 2005/04/03 16:44:55 jvunder Exp $
 *
 *   OpenChanfix v2.2 
 *
 *   Channel Re-op Service Module for ircd-ratbox-2.0.7.
 *   Copyright (C) 2003-2004 Thomas Carlsson and Joost Vunderink.
 *   See http://www.garion.org/ocf/ for more information.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include "cf_base.h"


mapi_hfn_list_av1 chanfix_hfnlist[] = {
	{"fake_message", (hookfn) catch_fake_msg},
	{"fake_kill", (hookfn) catch_chanfix_kill},
	{NULL, NULL}
};

void OCF_connect_chanfix();


static int
modinit(void)
{
	time(&module_loaded_time);
	current_day_since_epoch = get_days_since_epoch();

	currently_split = 1;
	send_notices_instead_of_privmsgs = 0;
	sent_init_message = 0;

	/* Init and load the settings */
	settings_init();
	ocf_log_init();
	settings_read_file();
	send_notices_instead_of_privmsgs = settings_get_bool("send_notices");
	ocf_logfiles_open();
	ocf_log(OCF_LOG_MAIN, "MAIN Loading OpenChanfix module.");
	ocf_users_init();
	ocf_ignores_init();

	/* Loads the database; will create a new one if no database
	 * can be found. */
	DB_load();
	DB_set_day(current_day_since_epoch);

	OCF_connect_chanfix();

	add_update_database_event();
	add_save_database_event();
	eventAdd("ocf_check_split",           &ocf_check_split, NULL, 10);
	eventAdd("ocf_check_opless_channels", &ocf_check_opless_channels, NULL, 10);
	eventAdd("ocf_fix_chanfix_channels",  &ocf_fix_chanfix_channels, NULL, 5);
	eventAdd("ocf_fix_autofix_channels",  &ocf_fix_autofix_channels, NULL, 5);
	eventAdd("ocf_check_loggedin_users", 
		&ocf_check_loggedin_users, NULL, OCF_USER_LOGGEDIN_INTERVAL * 60);

	ocf_user_load_all(settings_get_str("userfile"));
	ocf_user_load_loggedin();
	ocf_ignore_load(settings_get_str("ignorefile"));
	ocf_log(OCF_LOG_MAIN, "MAIN Initialisation complete.");
	return 1;
}

/* ----------------------------------------------------------------- */

static void
moddeinit(void)
{
	ocf_log(OCF_LOG_MAIN, "MAIN Unloading OpenChanfix module.");
	
	send_notice_to_loggedin(OCF_NOTICE_FLAGS_ALL, "Saving Database to disk.");
	DB_save();
	send_notice_to_loggedin(OCF_NOTICE_FLAGS_ALL, "Done. Unloaded OpenChanfix module.");

	ocf_user_save_loggedin();
	ocf_user_save_all(settings_get_str("userfile"));
	
	eventDelete(ocf_update_database, NULL);
	eventDelete(ocf_save_database, NULL);
	eventDelete(ocf_check_split, NULL);
	eventDelete(ocf_check_opless_channels, NULL);
	eventDelete(ocf_fix_chanfix_channels, NULL);
	eventDelete(ocf_fix_autofix_channels, NULL);
	eventDelete(ocf_check_loggedin_users, NULL);
	settings_deinit();
	ocf_users_deinit();
	ocf_logfiles_close();

	if (chanfix) {
		destroy_fake_client(chanfix);
	}
	
}

DECLARE_MODULE_AV1(ocf_module, modinit, moddeinit, NULL, NULL, chanfix_hfnlist, OCF_VERSION);


void OCF_connect_chanfix()
{

	chanfix = create_fake_client(settings_get_str("nick"), // nick
	                             settings_get_str("user"), // username 
	                             settings_get_str("host"), // hostname
	                             settings_get_str("real"), // real name (gecos)
	                             1); // opered client?
	
    ocf_log(OCF_LOG_DEBUG, "Connecting chanfix client.");

	return;
}

/* END OF CHANFIX initialisation module */
