/*
 * $Id: cf_message.c,v 1.1 2005/04/03 16:44:55 jvunder Exp $
 *
 *   OpenChanfix v2.2 
 *
 *   Channel Re-op Service Module for ircd-ratbox-2.0.7.
 *   Copyright (C) 2003-2004 Thomas Carlsson and Joost Vunderink.
 *   See http://www.garion.org/ocf/ for more information.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */



#include "cf_base.h"


/* ----------------------------------------------------------------- */

process_cmd_t process_cmd_functions[] = {
	process_cmd_unknown,
	process_cmd_help,
	process_cmd_score,
	process_cmd_chanfix,
	process_cmd_login,
	process_cmd_logout,
	process_cmd_passwd,
	process_cmd_history,
	process_cmd_info,
	process_cmd_oplist,
	process_cmd_addnote,
	process_cmd_delnote,
	process_cmd_whois,
	process_cmd_addflag,
	process_cmd_delflag,
	process_cmd_addhost,
	process_cmd_delhost,
	process_cmd_adduser,
	process_cmd_deluser,
	process_cmd_chpass,
	process_cmd_who,
	process_cmd_umode,
	process_cmd_status,
	process_cmd_set,
	process_cmd_block,
	process_cmd_unblock,
	process_cmd_rehash,
	process_cmd_check,
	process_cmd_opnicks,
	process_cmd_cscore,
	process_cmd_alert,
	process_cmd_unalert,
	process_cmd_addserver,
	process_cmd_delserver,
	process_cmd_chpasshash,
	process_cmd_whoserver,
	process_cmd_setmainserver,
};


/* ----------------------------------------------------------------- */

void send_notice_to_loggedin(int flag_required, char *pattern, ...)
{
	va_list args;
	char tmpBuf[IRCLINELEN];
	dlink_node *ptr, *next_ptr;
	struct ocf_loggedin_user *olu;
	int ver_user;

	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	tmpBuf[IRCLINELEN - 1] = 0;

	DLINK_FOREACH_SAFE(ptr, next_ptr, ocf_loggedin_user_list.head) {
		olu = ((struct ocf_loggedin_user*)(ptr->data));
		if (flag_required == OCF_NOTICE_FLAGS_ALL ||
				ocf_user_has_notice_flag(olu->ocf_user_p, flag_required)) {
			/* Make sure the client that currently has nickname olu->nick
			 * is the same as the one that logged in. */
			ver_user = verify_loggedin_user(olu);
			if (ver_user == 1) {
				chanfix_send_privmsg(olu->nick, "%s", tmpBuf);
			} else if (ver_user == 0 ) {
				/* Suppress the message, but don't logout the user. */	
			} else {
				ocf_user_logout(olu->nick);
			}
		}
	}
}

/* ----------------------------------------------------------------- */

void send_notice_to_loggedin_except(struct ocf_loggedin_user *user_p, 
							 int flag_required, char *pattern, ...)
{
	va_list args;
	char tmpBuf[IRCLINELEN];
	dlink_node *ptr, *next_ptr;
	struct ocf_loggedin_user *olu;
	int ver_user;

	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	tmpBuf[IRCLINELEN - 1] = 0;

	DLINK_FOREACH_SAFE(ptr, next_ptr, ocf_loggedin_user_list.head) {
		olu = ((struct ocf_loggedin_user*)(ptr->data));
		if ( (flag_required == OCF_NOTICE_FLAGS_ALL || 
				ocf_user_has_notice_flag(olu->ocf_user_p, flag_required) ) &&
				user_p != olu) {
			/* Make sure the client that currently has nickname olu->nick
			 * is the same as the one that logged in. */
			ver_user = verify_loggedin_user(olu);
			if (ver_user == 1) {
				chanfix_send_privmsg(olu->nick, "%s", tmpBuf);
			} else if (ver_user == 0 ) {
				/* Suppress the message, but don't logout the user. */	
			} else {
				ocf_user_logout(olu->nick);
			}
		}
	}
}

/* ----------------------------------------------------------------- */

void send_notice_to_loggedin_except_two(struct ocf_loggedin_user *user_p, 
							 struct ocf_loggedin_user *user2_p, 
							 int flag_required, char *pattern, ...)
{
	va_list args;
	char tmpBuf[IRCLINELEN];
	dlink_node *ptr, *next_ptr;
	struct ocf_loggedin_user *olu;
	int ver_user;

	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	tmpBuf[IRCLINELEN - 1] = 0;

	DLINK_FOREACH_SAFE(ptr, next_ptr, ocf_loggedin_user_list.head) {
		olu = ((struct ocf_loggedin_user*)(ptr->data));
		if ( (flag_required == OCF_NOTICE_FLAGS_ALL || 
				ocf_user_has_notice_flag(olu->ocf_user_p, flag_required) ) &&
				user_p != olu && user2_p != olu) {
			/* Make sure the client that currently has nickname olu->nick
			 * is the same as the one that logged in. */
			ver_user = verify_loggedin_user(olu);
			if (ver_user == 1) {
				chanfix_send_privmsg(olu->nick, "%s", tmpBuf);
			} else if (ver_user == 0 ) {
				/* Suppress the message, but don't logout the user. */	
			} else {
				ocf_user_logout(olu->nick);
			}
		}
	}
}

/* ----------------------------------------------------------------- */

void send_notice_to_loggedin_username(const char* username, char *pattern, ...)
{
	va_list args;
	char tmpBuf[IRCLINELEN];
	dlink_node *ptr;
	struct ocf_loggedin_user *olu;
	int ver_user;

	va_start(args, pattern);
	vsprintf(tmpBuf, pattern, args);
	va_end(args);
	tmpBuf[IRCLINELEN - 1] = 0;

	DLINK_FOREACH(ptr, ocf_loggedin_user_list.head) {
		olu = ((struct ocf_loggedin_user*)(ptr->data));
		if (!strcasecmp(olu->ocf_user_p->name, username)) {
			/* Make sure the client that currently has nickname olu->nick
			 * is the same as the one that logged in. */
			ver_user = verify_loggedin_user(olu);
			if (ver_user == 1) {
				chanfix_send_privmsg(olu->nick, "%s", tmpBuf);
			} else if (ver_user == 0 ) {
				/* Suppress the message, but don't logout the user. */	
			} else {
				ocf_user_logout(olu->nick);
			}
		}
	}
}
/* ----------------------------------------------------------------- */
/*
 *  A safer parser, returns 0 if incomplete/unreliable details were
 *  encountered, 1 if nick only, 3 if nick, user and host were
 *  successfully found.
 */
int parse_userhost_details( char *msg, struct message_data *md )
{
	int c;
	char* m[3] = { md->nick, md->user, md->host };
	int ml[3] = { NICKLEN, USERLEN, HOSTLEN };
	int mc[3] = { 0, 0, 0 };
	char* f;

	/* This check may seem a bit anal, but Mr Murphy loves it! */
	if ( !msg || !md ) return 0;

	for ( c = 0, f = msg; 1; f++ )
	{
		switch ( *f )
		{
			case ':':
			{
				/* Ignore colons */
				break;
			}
			case 0:
			case '\r':
			case '\n':
			{
				/* Something fishy with this string, let's disqualify */
				return 0;
			}
			case '!':
			{
				/* If we ran into the ! while filling the nick element,
				 * we'll move to user
				 */
				if ( c == 0 )
				{
					*m[0] = 0;
					c = 1;
				}
				else
				{
					/* This is unexpected, let's disqualify the whole string */
					return 0;
				}
				break;
			}
			case '@':
			{
				/* If we ran into the @ while filling the user element,
				 * we'll move to host.
				 */
				if ( c == 1 )
				{
					*m[1] = 0;
					c = 2;
				}
				else
				{
					/* This is unexpected, let's disqualify the whole string */
					return 0;
				}
				break;
			}
//			case 0:	/* Uncomment this to make parser more generic */
			case ' ':
			{
				/* Space at the end terminates parse */
				if ( mc[0] )
				{
 					m[c] = 0;
					return c + 1;
				}
				break;
			}
			default:
			{
				/* Let's make sure we're not about to overflow any structures,
				 * in case a compromised ircd sends us bad data.
				 */
				if ( mc[c] >= ml[c] )
				{
					/* Uh oh, going to overflow.. Let's disqualify the whole
					 * parse.
					 */
					*m[c] = 0;
					return 0;
				}

				/* By default, we just fill the active buffer */
				*m[c] = *f;
				m[c]++;
				mc[c]++;
			}
		}
	}
	/* Should never reach this point, but this will shut up compiler warnings */
	return 0;
}

int catch_fake_msg(struct h_fake_message_data *hd)
{
    struct message_data md;
    struct Client *client_p;
    
    if (!IsOper(hd->source_p))
        return 0;
    
    client_p = hd->target_p;
    if (client_p != chanfix)
        return 0;
        
    memset(&md, 0, sizeof(md));
    
    strlcpy(md.nick, hd->source_p->name, NICKLEN);
    strlcpy(md.user, hd->source_p->username, USERLEN);
    strlcpy(md.host, hd->source_p->host, HOSTLEN);
    strlcpy(md.server, hd->source_p->servptr->name, HOSTLEN);
    strlcpy(md.data, hd->message, IRCLINELEN);
    md.safe = hd->direct;
    
    process_privmsg(&md);    
    
    return 0; 
}

/* ----------------------------------------------------------------- */

int catch_chanfix_kill(struct Client *target_p)
{
	if(target_p != chanfix)
		return 0;

	ocf_log(OCF_LOG_MAIN, "KILL Chanfix client killed.");
	return 0;
}

/* ----------------------------------------------------------------- */

void process_privmsg(struct message_data *md)
{
	struct Client *sender = find_person(md->nick);
	struct ocf_loggedin_user *olu;
	struct command_data cd;
	process_cmd_t funcp;
	int required_flags;
	char hostmask[USERLEN + 1 + HOSTLEN + 1];
	
			
	/* Check if the sender is still on the network. */
	if (!sender) {
		/* Send error to owner(s) about non-existing nick */
		ocf_log(OCF_LOG_DEBUG, "pr_pr: nick %s does not exist.", md->nick);
		return;
	}

	/* Only accept commands from opers. */
	if (!IsOper(sender)) {
		/* Send error to owner(s) about non-oper command */
		ocf_log(OCF_LOG_DEBUG, "pr_pr: non-oper %s sent me %s", md->nick, md->data);
		ocf_log(OCF_LOG_PRIVMSG, "PRIV non-oper %s (%s@%s) [%s]: %s",
			md->nick, md->user, md->host, md->server, md->data);
		return;
	}

	snprintf(hostmask, sizeof(hostmask), "%s@%s", md->user, md->host);
    if (ocf_is_ignored(hostmask, md->server)) {
		ocf_log(OCF_LOG_PRIVMSG, "IGNO Ignoring message from oper %s (%s@%s) [%s]: %s",
			md->nick, md->user, md->host, md->server, md->data);
		return;
    }

	ocf_log(OCF_LOG_DEBUG, "Got privmsg from oper %s: %s", md->nick, md->data);
	ocf_log(OCF_LOG_PRIVMSG, "PRIV oper %s (%s@%s) [%s]: %s",
		md->nick, md->user, md->host, md->server, md->data);
	olu = ocf_get_loggedin_user(md->nick);

	/* Check if the user@host of the nickname that sent us this message is the
	 * same as the user@host of the logged in user. If not, log out that nickname`
	 * to avoid hijacking of logged in sessions. */
	if (olu != NULL) {
		if (strcasecmp(md->user, olu->user) != 0 || 
			strcasecmp(md->host, olu->host) != 0 ||
			strcasecmp(md->server, olu->server) != 0) {
			ocf_log(OCF_LOG_MAIN, 
				"WARN Possible session hijack for %s: logged in with %s@%s [%s], now %s@%s [%s].",
				md->nick, olu->user, olu->host, olu->server, md->user, md->host, md->server);
			send_notice_to_loggedin_except(olu, OCF_NOTICE_FLAGS_MAIN, 
				"Possible session hijack for %s: logged in with %s@%s [%s], now %s@%s [%s].",
				md->nick, olu->user, olu->host, olu->server, md->user, md->host, md->server);
			ocf_user_logout(md->nick);
			olu = NULL;
		}
	}

	cd.loggedin_user_p = olu;
	get_command_data(md->data, &cd);

	/* Check whether the user is logged in. */
	if (olu == NULL && command_requires_loggedin(cd.command)) {
		chanfix_send_privmsg(md->nick, "You need to login to use this command.");
		return;
	}

	/* Check whether the user has the necessary flags. */
	required_flags = cmd_flags_required[cd.command];
	if (olu && required_flags && !ocf_user_has_flag(olu->ocf_user_p, required_flags)) {
		if (ocf_user_flag_get_char(required_flags) != ' ') {
			chanfix_send_privmsg(md->nick, "This command requires flag %c.", 
				ocf_user_flag_get_char(required_flags));
		} else {
			chanfix_send_privmsg(md->nick, "This command requires one of these flags: %s.", 
				ocf_user_flags_to_string(required_flags));
		}
		return;
	}

	/* Check whether the syntax of the command is correct. */
	if (cd.wrong_syntax) {
		ocf_log(OCF_LOG_DEBUG, "Wrong syntax.");
		process_cmd_wrong_syntax(md, &cd);
		return;
	}

	/* Get the function pointer to the function to process this command. */
	funcp = process_cmd_functions[cd.command];
	if (funcp == NULL) {
		ocf_log(OCF_LOG_DEBUG, "Eeks! NULL function pointer in process_privmsg!");
		chanfix_send_privmsg(md->nick,
			"ERROR - NULL function pointer in process_privmsg for cmd %d. Please report to ocf@garion.org", cd.command);
		return;
	}
	funcp(md, &cd);
}

/* ----------------------------------------------------------------- */

void send_command_list(char* nick) {
	char cmds[IRCLINELEN];
	int i;
	struct ocf_loggedin_user *olu;
	cmds[0] = 0;
	olu = ocf_get_loggedin_user(nick);

	for (i = 1; i <= NUM_COMMANDS; i++) {
		/* Only send help for commands that require loggedin if the 
		 * user is actually logged in. */
		if (command_requires_loggedin(i)) {
			if (olu) {
				strcat(cmds, cmd_word[i]);
				if (i != NUM_COMMANDS) {
					strcat(cmds, " ");
				}
			}
		} else {
			strcat(cmds, cmd_word[i]);
			if (i != NUM_COMMANDS) {
				strcat(cmds, " ");
			}
		}
	}
	chanfix_send_privmsg(nick, "List of commands:");
	chanfix_send_privmsg(nick, "%s", cmds);
}

